include "config.pxi"
from api_types_hdf5 cimport *
from api_types_ext cimport *

cdef extern from "hdf5.h":
  herr_t H5open() except *
  herr_t H5close() except *
  herr_t H5get_libversion(unsigned *majnum, unsigned *minnum, unsigned *relnum) except *
  hid_t H5Dcreate2(hid_t loc_id, char *name, hid_t type_id, hid_t space_id, hid_t lcpl_id, hid_t dcpl_id, hid_t dapl_id) except *
  hid_t H5Dcreate_anon(hid_t file_id, hid_t type_id, hid_t space_id, hid_t plist_id, hid_t dapl_id) except *
  hid_t H5Dopen(hid_t file_id, char *name) except *
  herr_t H5Dclose(hid_t dset_id) except *
  hid_t H5Dget_space(hid_t dset_id) except *
  herr_t H5Dget_space_status(hid_t dset_id, H5D_space_status_t *status) except *
  hid_t H5Dget_type(hid_t dset_id) except *
  hid_t H5Dget_create_plist(hid_t dataset_id) except *
  haddr_t H5Dget_offset(hid_t dset_id) except *
  hsize_t H5Dget_storage_size(hid_t dset_id) except *
  herr_t H5Dread(hid_t dset_id, hid_t mem_type_id, hid_t mem_space_id, hid_t file_space_id, hid_t plist_id, void *buf) except *
  herr_t H5Dwrite(hid_t dset_id, hid_t mem_type, hid_t mem_space, hid_t file_space, hid_t xfer_plist, void* buf) except *
  herr_t H5Dextend(hid_t dataset_id, hsize_t *size) except *
  herr_t H5Dfill(void *fill, hid_t fill_type_id, void *buf,  hid_t buf_type_id, hid_t space_id ) except *
  herr_t H5Dvlen_get_buf_size(hid_t dset_id, hid_t type_id, hid_t space_id, hsize_t *size) except *
  herr_t H5Dvlen_reclaim(hid_t type_id, hid_t space_id,  hid_t plist, void *buf) except *
  herr_t H5Diterate(void *buf, hid_t type_id, hid_t space_id,  H5D_operator_t op, void* operator_data) except *
  herr_t H5Dset_extent(hid_t dset_id, hsize_t* size) except *
  hid_t H5Fcreate(char *filename, unsigned int flags, hid_t create_plist, hid_t access_plist) except *
  hid_t H5Fopen(char *name, unsigned flags, hid_t access_id) except *
  herr_t H5Fclose(hid_t file_id) except *
  htri_t H5Fis_hdf5(char *name) except *
  herr_t H5Fflush(hid_t object_id, H5F_scope_t scope) except *
  hid_t H5Freopen(hid_t file_id) except *
  herr_t H5Fmount(hid_t loc_id, char *name, hid_t child_id, hid_t plist_id) except *
  herr_t H5Funmount(hid_t loc_id, char *name) except *
  herr_t H5Fget_filesize(hid_t file_id, hsize_t *size) except *
  hid_t H5Fget_create_plist(hid_t file_id ) except *
  hid_t H5Fget_access_plist(hid_t file_id) except *
  hssize_t H5Fget_freespace(hid_t file_id) except *
  ssize_t H5Fget_name(hid_t obj_id, char *name, size_t size) except *
  int H5Fget_obj_count(hid_t file_id, unsigned int types) except *
  int H5Fget_obj_ids(hid_t file_id, unsigned int types, int max_objs, hid_t *obj_id_list) except *
  herr_t H5Fget_vfd_handle(hid_t file_id, hid_t fapl_id, void **file_handle) except *
  herr_t H5Fget_intent(hid_t file_id, unsigned int *intent) except *
  herr_t H5Fget_mdc_config(hid_t file_id, H5AC_cache_config_t *config_ptr) except *
  herr_t H5Fget_mdc_hit_rate(hid_t file_id, double *hit_rate_ptr) except *
  herr_t H5Fget_mdc_size(hid_t file_id, size_t *max_size_ptr, size_t *min_clean_size_ptr, size_t *cur_size_ptr, int *cur_num_entries_ptr) except *
  herr_t H5Freset_mdc_hit_rate_stats(hid_t file_id) except *
  herr_t H5Fset_mdc_config(hid_t file_id, H5AC_cache_config_t *config_ptr) except *
  hid_t H5Gcreate(hid_t loc_id, char *name, size_t size_hint) except *
  hid_t H5Gopen(hid_t loc_id, char *name) except *
  herr_t H5Gclose(hid_t group_id) except *
  herr_t H5Glink2( hid_t curr_loc_id, char *current_name, H5G_link_t link_type, hid_t new_loc_id, char *new_name) except *
  herr_t H5Gunlink(hid_t file_id, char *name) except *
  herr_t H5Gmove2(hid_t src_loc_id, char *src_name, hid_t dst_loc_id, char *dst_name) except *
  herr_t H5Gget_num_objs(hid_t loc_id, hsize_t*  num_obj) except *
  int H5Gget_objname_by_idx(hid_t loc_id, hsize_t idx, char *name, size_t size) except *
  int H5Gget_objtype_by_idx(hid_t loc_id, hsize_t idx) except *
  herr_t H5Giterate(hid_t loc_id, char *name, int *idx, H5G_iterate_t op, void* data) except *
  herr_t H5Gget_objinfo(hid_t loc_id, char* name, int follow_link, H5G_stat_t *statbuf) except *
  herr_t H5Gget_linkval(hid_t loc_id, char *name, size_t size, char *value) except *
  herr_t H5Gset_comment(hid_t loc_id, char *name, char *comment) except *
  int H5Gget_comment(hid_t loc_id, char *name, size_t bufsize, char *comment) except *
  hid_t H5Gcreate_anon( hid_t loc_id, hid_t gcpl_id, hid_t gapl_id) except *
  hid_t H5Gcreate2(hid_t loc_id, char *name, hid_t lcpl_id, hid_t gcpl_id, hid_t gapl_id) except *
  hid_t H5Gopen2( hid_t loc_id, char * name, hid_t gapl_id) except *
  herr_t H5Gget_info( hid_t group_id, H5G_info_t *group_info) except *
  herr_t H5Gget_info_by_name( hid_t loc_id, char *group_name, H5G_info_t *group_info, hid_t lapl_id) except *
  hid_t H5Gget_create_plist(hid_t group_id) except *
  H5I_type_t H5Iget_type(hid_t obj_id) except *
  ssize_t H5Iget_name( hid_t obj_id, char *name, size_t size) except *
  hid_t H5Iget_file_id(hid_t obj_id) except *
  int H5Idec_ref(hid_t obj_id) except *
  int H5Iget_ref(hid_t obj_id) except *
  int H5Iinc_ref(hid_t obj_id) except *
  herr_t H5Lmove(hid_t src_loc, char *src_name, hid_t dst_loc, char *dst_name, hid_t lcpl_id, hid_t lapl_id) except *
  herr_t H5Lcopy(hid_t src_loc, char *src_name, hid_t dst_loc, char *dst_name, hid_t lcpl_id, hid_t lapl_id) except *
  herr_t H5Lcreate_hard(hid_t cur_loc, char *cur_name, hid_t dst_loc, char *dst_name, hid_t lcpl_id, hid_t lapl_id) except *
  herr_t H5Lcreate_soft(char *link_target, hid_t link_loc_id, char *link_name, hid_t lcpl_id, hid_t lapl_id) except *
  herr_t H5Ldelete(hid_t loc_id, char *name, hid_t lapl_id) except *
  herr_t H5Ldelete_by_idx(hid_t loc_id, char *group_name, H5_index_t idx_type, H5_iter_order_t order, hsize_t n, hid_t lapl_id) except *
  herr_t H5Lget_val(hid_t loc_id, char *name, void *bufout, size_t size, hid_t lapl_id) except *
  herr_t H5Lget_val_by_idx(hid_t loc_id, char *group_name,  H5_index_t idx_type, H5_iter_order_t order, hsize_t n, void *bufout, size_t size, hid_t lapl_id) except *
  htri_t H5Lexists(hid_t loc_id, char *name, hid_t lapl_id) except *
  herr_t H5Lget_info(hid_t loc_id, char *name, H5L_info_t *linfo, hid_t lapl_id) except *
  herr_t H5Lget_info_by_idx(hid_t loc_id, char *group_name, H5_index_t idx_type, H5_iter_order_t order, hsize_t n, H5L_info_t *linfo, hid_t lapl_id) except *
  ssize_t H5Lget_name_by_idx(hid_t loc_id, char *group_name, H5_index_t idx_type, H5_iter_order_t order, hsize_t n, char *name, size_t size, hid_t lapl_id) except *
  herr_t H5Literate(hid_t grp_id, H5_index_t idx_type, H5_iter_order_t order, hsize_t *idx, H5L_iterate_t op, void *op_data) except *
  herr_t H5Literate_by_name(hid_t loc_id, char *group_name, H5_index_t idx_type, H5_iter_order_t order, hsize_t *idx, H5L_iterate_t op, void *op_data, hid_t lapl_id) except *
  herr_t H5Lvisit(hid_t grp_id, H5_index_t idx_type, H5_iter_order_t order, H5L_iterate_t op, void *op_data) except *
  herr_t H5Lvisit_by_name(hid_t loc_id, char *group_name, H5_index_t idx_type, H5_iter_order_t order, H5L_iterate_t op, void *op_data, hid_t lapl_id) except *
  herr_t H5Lunpack_elink_val(void *ext_linkval, size_t link_size, unsigned *flags, char **filename, char **obj_path) except *
  herr_t H5Lcreate_external(char *file_name, char *obj_name, hid_t link_loc_id, char *link_name, hid_t lcpl_id, hid_t lapl_id) except *
  hid_t H5Oopen(hid_t loc_id, char *name, hid_t lapl_id) except *
  hid_t H5Oopen_by_addr(hid_t loc_id, haddr_t addr) except *
  hid_t H5Oopen_by_idx(hid_t loc_id, char *group_name, H5_index_t idx_type, H5_iter_order_t order, hsize_t n, hid_t lapl_id) except *
  herr_t H5Oget_info(hid_t loc_id, H5O_info_t *oinfo) except *
  herr_t H5Oget_info_by_name(hid_t loc_id, char *name, H5O_info_t *oinfo, hid_t lapl_id) except *
  herr_t H5Oget_info_by_idx(hid_t loc_id, char *group_name,  H5_index_t idx_type, H5_iter_order_t order, hsize_t n, H5O_info_t *oinfo, hid_t lapl_id) except *
  herr_t H5Olink(hid_t obj_id, hid_t new_loc_id, char *new_name, hid_t lcpl_id, hid_t lapl_id) except *
  herr_t H5Ocopy(hid_t src_loc_id, char *src_name, hid_t dst_loc_id,  char *dst_name, hid_t ocpypl_id, hid_t lcpl_id) except *
  herr_t H5Oincr_refcount(hid_t object_id) except *
  herr_t H5Odecr_refcount(hid_t object_id) except *
  herr_t H5Oset_comment(hid_t obj_id, char *comment) except *
  herr_t H5Oset_comment_by_name(hid_t loc_id, char *name,  char *comment, hid_t lapl_id) except *
  ssize_t H5Oget_comment(hid_t obj_id, char *comment, size_t bufsize) except *
  ssize_t H5Oget_comment_by_name(hid_t loc_id, char *name, char *comment, size_t bufsize, hid_t lapl_id) except *
  herr_t H5Ovisit(hid_t obj_id, H5_index_t idx_type, H5_iter_order_t order,  H5O_iterate_t op, void *op_data) except *
  herr_t H5Ovisit_by_name(hid_t loc_id, char *obj_name, H5_index_t idx_type, H5_iter_order_t order, H5O_iterate_t op, void *op_data, hid_t lapl_id) except *
  herr_t H5Oclose(hid_t object_id) except *
  hid_t H5Pcreate(hid_t plist_id) except *
  hid_t H5Pcopy(hid_t plist_id) except *
  int H5Pget_class(hid_t plist_id) except *
  herr_t H5Pclose(hid_t plist_id) except *
  htri_t H5Pequal( hid_t id1, hid_t id2 ) except *
  herr_t H5Pclose_class(hid_t id) except *
  herr_t H5Pget_version(hid_t plist, unsigned int *super_, unsigned int* freelist,  unsigned int *stab, unsigned int *shhdr) except *
  herr_t H5Pset_userblock(hid_t plist, hsize_t size) except *
  herr_t H5Pget_userblock(hid_t plist, hsize_t * size) except *
  herr_t H5Pset_sizes(hid_t plist, size_t sizeof_addr, size_t sizeof_size) except *
  herr_t H5Pget_sizes(hid_t plist, size_t *sizeof_addr, size_t *sizeof_size) except *
  herr_t H5Pset_sym_k(hid_t plist, unsigned int ik, unsigned int lk) except *
  herr_t H5Pget_sym_k(hid_t plist, unsigned int *ik, unsigned int *lk) except *
  herr_t H5Pset_istore_k(hid_t plist, unsigned int ik) except *
  herr_t H5Pget_istore_k(hid_t plist, unsigned int *ik) except *
  herr_t H5Pset_fclose_degree(hid_t fapl_id, H5F_close_degree_t fc_degree) except *
  herr_t H5Pget_fclose_degree(hid_t fapl_id, H5F_close_degree_t *fc_degree) except *
  herr_t H5Pset_fapl_core( hid_t fapl_id, size_t increment, hbool_t backing_store) except *
  herr_t H5Pget_fapl_core( hid_t fapl_id, size_t *increment, hbool_t *backing_store) except *
  herr_t H5Pset_fapl_family( hid_t fapl_id,  hsize_t memb_size, hid_t memb_fapl_id ) except *
  herr_t H5Pget_fapl_family( hid_t fapl_id, hsize_t *memb_size, hid_t *memb_fapl_id ) except *
  herr_t H5Pset_family_offset( hid_t fapl_id, hsize_t offset) except *
  herr_t H5Pget_family_offset( hid_t fapl_id, hsize_t *offset) except *
  herr_t H5Pset_fapl_log(hid_t fapl_id, char *logfile, unsigned int flags, size_t buf_size) except *
  herr_t H5Pset_fapl_multi(hid_t fapl_id, H5FD_mem_t *memb_map, hid_t *memb_fapl, char **memb_name, haddr_t *memb_addr, hbool_t relax) except *
  herr_t H5Pset_cache(hid_t plist_id, int mdc_nelmts, int rdcc_nelmts,  size_t rdcc_nbytes, double rdcc_w0) except *
  herr_t H5Pget_cache(hid_t plist_id, int *mdc_nelmts, int *rdcc_nelmts, size_t *rdcc_nbytes, double *rdcc_w0) except *
  herr_t H5Pset_fapl_sec2(hid_t fapl_id) except *
  herr_t H5Pset_fapl_stdio(hid_t fapl_id) except *
  hid_t H5Pget_driver(hid_t fapl_id) except *
  herr_t H5Pget_mdc_config(hid_t plist_id, H5AC_cache_config_t *config_ptr) except *
  herr_t H5Pset_mdc_config(hid_t plist_id, H5AC_cache_config_t *config_ptr) except *
  herr_t H5Pset_layout(hid_t plist, int layout) except *
  H5D_layout_t H5Pget_layout(hid_t plist) except *
  herr_t H5Pset_chunk(hid_t plist, int ndims, hsize_t * dim) except *
  int H5Pget_chunk(hid_t plist, int max_ndims, hsize_t * dims ) except *
  herr_t H5Pset_deflate( hid_t plist, int level) except *
  herr_t H5Pset_fill_value(hid_t plist_id, hid_t type_id, void *value ) except *
  herr_t H5Pget_fill_value(hid_t plist_id, hid_t type_id, void *value ) except *
  herr_t H5Pfill_value_defined(hid_t plist_id, H5D_fill_value_t *status ) except *
  herr_t H5Pset_fill_time(hid_t plist_id, H5D_fill_time_t fill_time ) except *
  herr_t H5Pget_fill_time(hid_t plist_id, H5D_fill_time_t *fill_time ) except *
  herr_t H5Pset_alloc_time(hid_t plist_id, H5D_alloc_time_t alloc_time ) except *
  herr_t H5Pget_alloc_time(hid_t plist_id, H5D_alloc_time_t *alloc_time ) except *
  herr_t H5Pset_filter(hid_t plist, H5Z_filter_t filter, unsigned int flags, size_t cd_nelmts, unsigned int* cd_values ) except *
  htri_t H5Pall_filters_avail(hid_t dcpl_id) except *
  int H5Pget_nfilters(hid_t plist) except *
  H5Z_filter_t H5Pget_filter(hid_t plist, unsigned int filter_number,   unsigned int *flags, size_t *cd_nelmts,  unsigned int* cd_values, size_t namelen, char* name ) except *
  herr_t H5Pget_filter_by_id( hid_t plist_id, H5Z_filter_t filter,  unsigned int *flags, size_t *cd_nelmts,  unsigned int* cd_values, size_t namelen, char* name) except *
  herr_t H5Pmodify_filter(hid_t plist, H5Z_filter_t filter, unsigned int flags, size_t cd_nelmts, unsigned int *cd_values) except *
  herr_t H5Premove_filter(hid_t plist, H5Z_filter_t filter ) except *
  herr_t H5Pset_fletcher32(hid_t plist) except *
  herr_t H5Pset_shuffle(hid_t plist_id) except *
  herr_t H5Pset_szip(hid_t plist, unsigned int options_mask, unsigned int pixels_per_block) except *
  herr_t H5Pset_scaleoffset(hid_t plist, H5Z_SO_scale_type_t scale_type, int scale_factor) except *
  herr_t H5Pset_edc_check(hid_t plist, H5Z_EDC_t check) except *
  H5Z_EDC_t H5Pget_edc_check(hid_t plist) except *
  herr_t H5Pset_chunk_cache( hid_t dapl_id, size_t rdcc_nslots, size_t rdcc_nbytes, double rdcc_w0 ) except *
  herr_t H5Pget_chunk_cache( hid_t dapl_id, size_t *rdcc_nslots, size_t *rdcc_nbytes, double *rdcc_w0 ) except *
  herr_t H5Pset_sieve_buf_size(hid_t fapl_id, size_t size) except *
  herr_t H5Pget_sieve_buf_size(hid_t fapl_id, size_t *size) except *
  herr_t H5Pset_nlinks(hid_t plist_id, size_t nlinks) except *
  herr_t H5Pget_nlinks(hid_t plist_id, size_t *nlinks) except *
  herr_t H5Pset_elink_prefix(hid_t plist_id, char *prefix) except *
  ssize_t H5Pget_elink_prefix(hid_t plist_id, char *prefix, size_t size) except *
  hid_t H5Pget_elink_fapl(hid_t lapl_id) except *
  herr_t H5Pset_elink_fapl(hid_t lapl_id, hid_t fapl_id) except *
  herr_t H5Pset_create_intermediate_group(hid_t plist_id, unsigned crt_intmd) except *
  herr_t H5Pget_create_intermediate_group(hid_t plist_id, unsigned *crt_intmd) except *
  herr_t H5Pset_copy_object(hid_t plist_id, unsigned crt_intmd) except *
  herr_t H5Pget_copy_object(hid_t plist_id, unsigned *crt_intmd) except *
  herr_t H5Pset_char_encoding(hid_t plist_id, H5T_cset_t encoding) except *
  herr_t H5Pget_char_encoding(hid_t plist_id, H5T_cset_t *encoding) except *
  herr_t H5Pset_obj_track_times( hid_t ocpl_id, hbool_t track_times ) except *
  herr_t H5Pget_obj_track_times( hid_t ocpl_id, hbool_t *track_times ) except *
  herr_t H5Pset_local_heap_size_hint(hid_t plist_id, size_t size_hint) except *
  herr_t H5Pget_local_heap_size_hint(hid_t plist_id, size_t *size_hint) except *
  herr_t H5Pset_link_phase_change(hid_t plist_id, unsigned max_compact, unsigned min_dense) except *
  herr_t H5Pget_link_phase_change(hid_t plist_id, unsigned *max_compact , unsigned *min_dense) except *
  herr_t H5Pset_est_link_info(hid_t plist_id, unsigned est_num_entries, unsigned est_name_len) except *
  herr_t H5Pget_est_link_info(hid_t plist_id, unsigned *est_num_entries , unsigned *est_name_len) except *
  herr_t H5Pset_link_creation_order(hid_t plist_id, unsigned crt_order_flags) except *
  herr_t H5Pget_link_creation_order(hid_t plist_id, unsigned *crt_order_flags) except *
  herr_t H5Pset_libver_bounds(hid_t fapl_id, H5F_libver_t libver_low, H5F_libver_t libver_high) except *
  herr_t H5Pget_libver_bounds(hid_t fapl_id, H5F_libver_t *libver_low, H5F_libver_t *libver_high) except *
  herr_t H5Rcreate(void *ref, hid_t loc_id, char *name, H5R_type_t ref_type,  hid_t space_id) except *
  hid_t H5Rdereference(hid_t obj_id, H5R_type_t ref_type, void *ref) except *
  hid_t H5Rget_region(hid_t dataset, H5R_type_t ref_type, void *ref) except *
  H5G_obj_t H5Rget_obj_type(hid_t id, H5R_type_t ref_type, void *ref) except *
  ssize_t H5Rget_name(hid_t loc_id, H5R_type_t ref_type, void *ref, char *name, size_t size) except *
  hid_t H5Screate(H5S_class_t type) except *
  hid_t H5Scopy(hid_t space_id ) except *
  herr_t H5Sclose(hid_t space_id) except *
  hid_t H5Screate_simple(int rank, hsize_t *dims, hsize_t *maxdims) except *
  htri_t H5Sis_simple(hid_t space_id) except *
  herr_t H5Soffset_simple(hid_t space_id, hssize_t *offset ) except *
  int H5Sget_simple_extent_ndims(hid_t space_id) except *
  int H5Sget_simple_extent_dims(hid_t space_id, hsize_t *dims, hsize_t *maxdims) except *
  hssize_t H5Sget_simple_extent_npoints(hid_t space_id) except *
  H5S_class_t H5Sget_simple_extent_type(hid_t space_id) except *
  herr_t H5Sextent_copy(hid_t dest_space_id, hid_t source_space_id ) except *
  herr_t H5Sset_extent_simple(hid_t space_id, int rank, hsize_t *current_size, hsize_t *maximum_size ) except *
  herr_t H5Sset_extent_none(hid_t space_id) except *
  H5S_sel_type H5Sget_select_type(hid_t space_id) except *
  hssize_t H5Sget_select_npoints(hid_t space_id) except *
  herr_t H5Sget_select_bounds(hid_t space_id, hsize_t *start, hsize_t *end) except *
  herr_t H5Sselect_all(hid_t space_id) except *
  herr_t H5Sselect_none(hid_t space_id) except *
  htri_t H5Sselect_valid(hid_t space_id) except *
  hssize_t H5Sget_select_elem_npoints(hid_t space_id) except *
  herr_t H5Sget_select_elem_pointlist(hid_t space_id, hsize_t startpoint,  hsize_t numpoints, hsize_t *buf) except *
  herr_t H5Sselect_elements(hid_t space_id, H5S_seloper_t op,  size_t num_elements, hsize_t **coord) except *
  hssize_t H5Sget_select_hyper_nblocks(hid_t space_id ) except *
  herr_t H5Sget_select_hyper_blocklist(hid_t space_id,  hsize_t startblock, hsize_t numblocks, hsize_t *buf ) except *
  herr_t H5Sselect_hyperslab(hid_t space_id, H5S_seloper_t op,  hsize_t *start, hsize_t *_stride, hsize_t *count, hsize_t *_block) except *
  herr_t H5Sencode(hid_t obj_id, void *buf, size_t *nalloc) except *
  hid_t H5Sdecode(void *buf) except *
  hid_t H5Tcreate(H5T_class_t type, size_t size) except *
  hid_t H5Topen(hid_t loc, char* name) except *
  herr_t H5Tcommit(hid_t loc_id, char* name, hid_t type) except *
  htri_t H5Tcommitted(hid_t type) except *
  hid_t H5Tcopy(hid_t type_id) except *
  htri_t H5Tequal(hid_t type_id1, hid_t type_id2 ) except *
  herr_t H5Tlock(hid_t type_id) except *
  H5T_class_t H5Tget_class(hid_t type_id) except *
  size_t H5Tget_size(hid_t type_id) except *
  hid_t H5Tget_super(hid_t type) except *
  htri_t H5Tdetect_class(hid_t type_id, H5T_class_t dtype_class) except *
  herr_t H5Tclose(hid_t type_id) except *
  hid_t H5Tget_native_type(hid_t type_id, H5T_direction_t direction) except *
  herr_t H5Tconvert(hid_t src_id, hid_t dst_id, size_t nelmts, void *buf, void *background, hid_t plist_id) except *
  herr_t H5Tset_size(hid_t type_id, size_t size) except *
  H5T_order_t H5Tget_order(hid_t type_id) except *
  herr_t H5Tset_order(hid_t type_id, H5T_order_t order) except *
  hsize_t H5Tget_precision(hid_t type_id) except *
  herr_t H5Tset_precision(hid_t type_id, size_t prec) except *
  int H5Tget_offset(hid_t type_id) except *
  herr_t H5Tset_offset(hid_t type_id, size_t offset) except *
  herr_t H5Tget_pad(hid_t type_id, H5T_pad_t * lsb, H5T_pad_t * msb ) except *
  herr_t H5Tset_pad(hid_t type_id, H5T_pad_t lsb, H5T_pad_t msb ) except *
  H5T_sign_t H5Tget_sign(hid_t type_id) except *
  herr_t H5Tset_sign(hid_t type_id, H5T_sign_t sign) except *
  herr_t H5Tget_fields(hid_t type_id, size_t *spos, size_t *epos, size_t *esize, size_t *mpos, size_t *msize ) except *
  herr_t H5Tset_fields(hid_t type_id, size_t spos, size_t epos, size_t esize, size_t mpos, size_t msize ) except *
  size_t H5Tget_ebias(hid_t type_id) except *
  herr_t H5Tset_ebias(hid_t type_id, size_t ebias) except *
  H5T_norm_t H5Tget_norm(hid_t type_id) except *
  herr_t H5Tset_norm(hid_t type_id, H5T_norm_t norm) except *
  H5T_pad_t H5Tget_inpad(hid_t type_id) except *
  herr_t H5Tset_inpad(hid_t type_id, H5T_pad_t inpad) except *
  H5T_cset_t H5Tget_cset(hid_t type_id) except *
  herr_t H5Tset_cset(hid_t type_id, H5T_cset_t cset) except *
  H5T_str_t H5Tget_strpad(hid_t type_id) except *
  herr_t H5Tset_strpad(hid_t type_id, H5T_str_t strpad) except *
  hid_t H5Tvlen_create(hid_t base_type_id) except *
  htri_t H5Tis_variable_str(hid_t dtype_id) except *
  int H5Tget_nmembers(hid_t type_id) except *
  H5T_class_t H5Tget_member_class(hid_t type_id, int member_no) except *
  char* H5Tget_member_name(hid_t type_id, unsigned membno) except *
  hid_t H5Tget_member_type(hid_t type_id, unsigned membno) except *
  int H5Tget_member_offset(hid_t type_id, int membno) except *
  int H5Tget_member_index(hid_t type_id, char* name) except *
  herr_t H5Tinsert(hid_t parent_id, char *name, size_t offset, hid_t member_id) except *
  herr_t H5Tpack(hid_t type_id) except *
  hid_t H5Tenum_create(hid_t base_id) except *
  herr_t H5Tenum_insert(hid_t type, char *name, void *value) except *
  herr_t H5Tenum_nameof( hid_t type, void *value, char *name, size_t size ) except *
  herr_t H5Tenum_valueof( hid_t type, char *name, void *value ) except *
  herr_t H5Tget_member_value(hid_t type,  unsigned int memb_no, void *value ) except *
  hid_t H5Tarray_create(hid_t base_id, int ndims, hsize_t *dims, int *perm) except *
  int H5Tget_array_ndims(hid_t type_id) except *
  int H5Tget_array_dims(hid_t type_id, hsize_t *dims, int *perm) except *
  herr_t H5Tset_tag(hid_t type_id, char* tag) except *
  char* H5Tget_tag(hid_t type_id) except *
  hid_t H5Tdecode(unsigned char *buf) except *
  herr_t H5Tencode(hid_t obj_id, unsigned char *buf, size_t *nalloc) except *
  herr_t H5Tcommit2(hid_t loc_id, char *name, hid_t dtype_id, hid_t lcpl_id, hid_t tcpl_id, hid_t tapl_id) except *
  H5T_conv_t H5Tfind(hid_t src_id, hid_t dst_id, H5T_cdata_t **pcdata) except *
  herr_t H5Tregister(H5T_pers_t pers, char *name, hid_t src_id, hid_t dst_id, H5T_conv_t func) except *
  herr_t H5Tunregister(H5T_pers_t pers, char *name, hid_t src_id, hid_t dst_id, H5T_conv_t func) except *
  htri_t H5Zfilter_avail(H5Z_filter_t id_) except *
  herr_t H5Zget_filter_info(H5Z_filter_t filter_, unsigned int *filter_config_flags) except *
  hid_t H5Acreate(hid_t loc_id, char *name, hid_t type_id, hid_t space_id, hid_t create_plist) except *
  hid_t H5Aopen_idx(hid_t loc_id, unsigned int idx) except *
  hid_t H5Aopen_name(hid_t loc_id, char *name) except *
  herr_t H5Aclose(hid_t attr_id) except *
  herr_t H5Adelete(hid_t loc_id, char *name) except *
  herr_t H5Aread(hid_t attr_id, hid_t mem_type_id, void *buf) except *
  herr_t H5Awrite(hid_t attr_id, hid_t mem_type_id, void *buf ) except *
  int H5Aget_num_attrs(hid_t loc_id) except *
  ssize_t H5Aget_name(hid_t attr_id, size_t buf_size, char *buf) except *
  hid_t H5Aget_space(hid_t attr_id) except *
  hid_t H5Aget_type(hid_t attr_id) except *
  herr_t H5Aiterate(hid_t loc_id, unsigned * idx, H5A_operator_t op, void* op_data) except *
  herr_t H5Adelete_by_name(hid_t loc_id, char *obj_name, char *attr_name, hid_t lapl_id) except *
  herr_t H5Adelete_by_idx(hid_t loc_id, char *obj_name, H5_index_t idx_type, H5_iter_order_t order, hsize_t n, hid_t lapl_id) except *
  hid_t H5Acreate_by_name(hid_t loc_id, char *obj_name, char *attr_name, hid_t type_id, hid_t space_id, hid_t acpl_id, hid_t aapl_id, hid_t lapl_id) except *
  herr_t H5Aopen(hid_t obj_id, char *attr_name, hid_t aapl_id) except *
  herr_t H5Aopen_by_name( hid_t loc_id, char *obj_name, char *attr_name, hid_t aapl_id, hid_t lapl_id) except *
  herr_t H5Aopen_by_idx(hid_t loc_id, char *obj_name, H5_index_t idx_type, H5_iter_order_t order, hsize_t n, hid_t aapl_id, hid_t lapl_id) except *
  htri_t H5Aexists_by_name( hid_t loc_id, char *obj_name, char *attr_name, hid_t lapl_id) except *
  htri_t H5Aexists(hid_t obj_id, char *attr_name) except *
  herr_t H5Arename(hid_t loc_id, char *old_attr_name, char *new_attr_name) except *
  herr_t H5Arename_by_name(hid_t loc_id, char *obj_name, char *old_attr_name, char *new_attr_name, hid_t lapl_id) except *
  herr_t H5Aget_info( hid_t attr_id, H5A_info_t *ainfo) except *
  herr_t H5Aget_info_by_name(hid_t loc_id, char *obj_name, char *attr_name, H5A_info_t *ainfo, hid_t lapl_id) except *
  herr_t H5Aget_info_by_idx(hid_t loc_id, char *obj_name, H5_index_t idx_type, H5_iter_order_t order, hsize_t n, H5A_info_t *ainfo, hid_t lapl_id) except *
  herr_t H5Aiterate2(hid_t obj_id, H5_index_t idx_type, H5_iter_order_t order, hsize_t *n, H5A_operator2_t op, void *op_data) except *
  hsize_t H5Aget_storage_size(hid_t attr_id) except *
  IF MPI:
    herr_t H5Pset_fapl_mpio(hid_t fapl_id, MPI_Comm comm, MPI_Info info) except *
  IF MPI:
    herr_t H5Pset_fapl_mpiposix(hid_t fapl_id, MPI_Comm comm, hbool_t hints) except *
  IF HDF5_VERSION >= (1, 8, 9):
    IF MPI:
        herr_t H5Fset_mpi_atomicity(hid_t file_id, hbool_t flag) except *
  IF HDF5_VERSION >= (1, 8, 9):
    IF MPI:
        herr_t H5Fget_mpi_atomicity(hid_t file_id, hbool_t *flag) except *
cdef extern from "hdf5_hl.h":
  herr_t H5DSattach_scale(hid_t did, hid_t dsid, unsigned int idx) except *
  herr_t H5DSdetach_scale(hid_t did, hid_t dsid, unsigned int idx) except *
  herr_t H5DSset_scale(hid_t dsid, char *dimname) except *
  int H5DSget_num_scales(hid_t did, unsigned int dim) except *
  herr_t H5DSset_label(hid_t did, unsigned int idx, char *label) except *
  ssize_t H5DSget_label(hid_t did, unsigned int idx, char *label, size_t size) except *
  ssize_t H5DSget_scale_name(hid_t did, char *name, size_t size) except *
  htri_t H5DSis_scale(hid_t did) except *
  herr_t H5DSiterate_scales(hid_t did, unsigned int dim, int *idx, H5DS_iterate_t visitor, void *visitor_data) except *
  htri_t H5DSis_attached(hid_t did, hid_t dsid, unsigned int idx) except *



# This file is part of h5py, a Python interface to the HDF5 library.
#
# http://www.h5py.org
#
# Copyright 2008-2013 Andrew Collette and contributors
#
# License:  Standard 3-clause BSD; see "license.txt" for full license terms
#           and contributor agreement.

import numpy
import collections

import h5py
from h5py import h5s, h5t, h5a
from . import base
from .dataset import readtime_dtype


class AttributeManager(base.DictCompat, base.CommonStateObject):

    """
        Allows dictionary-style access to an HDF5 object's attributes.

        These are created exclusively by the library and are available as
        a Python attribute at <object>.attrs

        Like Group objects, attributes provide a minimal dictionary-
        style interface.  Anything which can be reasonably converted to a
        Numpy array or Numpy scalar can be stored.

        Attributes are automatically created on assignment with the
        syntax <obj>.attrs[name] = value, with the HDF5 type automatically
        deduced from the value.  Existing attributes are overwritten.

        To modify an existing attribute while preserving its type, use the
        method modify().  To specify an attribute of a particular type and
        shape, use create().
    """

    def __init__(self, parent):
        """ Private constructor.
        """
        self._id = parent.id

    def __getitem__(self, name):
        """ Read the value of an attribute.
        """
        attr = h5a.open(self._id, self._e(name))

        if attr.get_space().get_simple_extent_type() == h5s.NULL:
            raise IOError("Empty attributes cannot be read")

        tid = attr.get_type()

        rtdt = readtime_dtype(attr.dtype, [])

        arr = numpy.ndarray(attr.shape, dtype=rtdt, order='C')
        attr.read(arr)

        if len(arr.shape) == 0:
            return arr[()]
        return arr

    def __setitem__(self, name, value):
        """ Set a new attribute, overwriting any existing attribute.

        The type and shape of the attribute are determined from the data.  To
        use a specific type or shape, or to preserve the type of an attribute,
        use the methods create() and modify().
        """
        self.create(name, data=value, dtype=base.guess_dtype(value))

    def __delitem__(self, name):
        """ Delete an attribute (which must already exist). """
        h5a.delete(self._id, self._e(name))

    def create(self, name, data, shape=None, dtype=None):
        """ Create a new attribute, overwriting any existing attribute.

        name
            Name of the new attribute (required)
        data
            An array to initialize the attribute (required)
        shape
            Shape of the attribute.  Overrides data.shape if both are
            given, in which case the total number of points must be unchanged.
        dtype
            Data type of the attribute.  Overrides data.dtype if both
            are given.
        """

        if data is not None:
            data = numpy.asarray(data, order='C', dtype=dtype)
            if shape is None:
                shape = data.shape
            elif numpy.product(shape) != numpy.product(data.shape):
                raise ValueError("Shape of new attribute conflicts with shape of data")

            if dtype is None:
                dtype = data.dtype

        if isinstance(dtype, h5py.Datatype):
            htype = dtype.id
            dtype = htype.dtype
        else:
            if dtype is None:
                dtype = numpy.dtype('f')
            htype = h5t.py_create(dtype, logical=True)

        if shape is None:
            raise ValueError('At least one of "shape" or "data" must be given')

        data = data.reshape(shape)

        space = h5s.create_simple(shape)

        if name in self:
            h5a.delete(self._id, self._e(name))

        attr = h5a.create(self._id, self._e(name), htype, space)

        if data is not None:
            try:
                attr.write(data)
            except:
                attr._close()
                h5a.delete(self._id, self._e(name))
                raise

    def modify(self, name, value):
        """ Change the value of an attribute while preserving its type.

        Differs from __setitem__ in that if the attribute already exists, its
        type is preserved.  This can be very useful for interacting with
        externally generated files.

        If the attribute doesn't exist, it will be automatically created.
        """
        if not name in self:
            self[name] = value
        else:
            value = numpy.asarray(value, order='C')

            attr = h5a.open(self._id, self._e(name))

            if attr.get_space().get_simple_extent_type() == h5s.NULL:
                raise IOError("Empty attributes can't be modified")

            # Allow the case of () <-> (1,)
            if (value.shape != attr.shape) and not \
               (numpy.product(value.shape) == 1 and numpy.product(attr.shape) == 1):
                raise TypeError("Shape of data is incompatible with existing attribute")
            attr.write(value)

    def __len__(self):
        """ Number of attributes attached to the object. """
        # I expect we will not have more than 2**32 attributes
        return h5a.get_num_attrs(self._id)

    def __iter__(self):
        """ Iterate over the names of attributes. """
        attrlist = []

        def iter_cb(name, *args):
            attrlist.append(self._d(name))
        h5a.iterate(self._id, iter_cb)

        for name in attrlist:
            yield name

    def __contains__(self, name):
        """ Determine if an attribute exists, by name. """
        return h5a.exists(self._id, self._e(name))

    def __repr__(self):
        if not self._id:
            return "<Attributes of closed HDF5 object>"
        return "<Attributes of HDF5 object at %s>" % id(self._id)

collections.MutableMapping.register(AttributeManager)



# This file is part of h5py, a Python interface to the HDF5 library.
#
# http://www.h5py.org
#
# Copyright 2008-2013 Andrew Collette and contributors
#
# License:  Standard 3-clause BSD; see "license.txt" for full license terms
#           and contributor agreement.

import weakref
import sys
import os

from .base import HLObject, py3
from .group import Group
from h5py import h5, h5f, h5p, h5i, h5fd, h5t, _objects
from h5py import version

mpi = h5.get_config().mpi
hdf5_version = version.hdf5_version_tuple[0:3]

if mpi:
    import mpi4py

libver_dict = {'earliest': h5f.LIBVER_EARLIEST, 'latest': h5f.LIBVER_LATEST}
libver_dict_r = dict((y, x) for x, y in libver_dict.iteritems())


def make_fapl(driver, libver, **kwds):
    """ Set up a file access property list """
    plist = h5p.create(h5p.FILE_ACCESS)
    plist.set_fclose_degree(h5f.CLOSE_STRONG)

    if libver is not None:
        if libver in libver_dict:
            low = libver_dict[libver]
            high = h5f.LIBVER_LATEST
        else:
            low, high = (libver_dict[x] for x in libver)
        plist.set_libver_bounds(low, high)

    if driver is None or (driver == 'windows' and sys.platform == 'win32'):
        return plist

    if(driver == 'sec2'):
        plist.set_fapl_sec2(**kwds)
    elif(driver == 'stdio'):
        plist.set_fapl_stdio(**kwds)
    elif(driver == 'core'):
        plist.set_fapl_core(**kwds)
    elif(driver == 'family'):
        plist.set_fapl_family(memb_fapl=plist.copy(), **kwds)
    elif(driver == 'mpio'):
        kwds.setdefault('info', mpi4py.MPI.Info())
        plist.set_fapl_mpio(**kwds)
    else:
        raise ValueError('Unknown driver type "%s"' % driver)

    return plist


def make_fid(name, mode, userblock_size, fapl, fcpl=None):
    """ Get a new FileID by opening or creating a file.
    Also validates mode argument."""

    if userblock_size is not None:
        if mode in ('r', 'r+'):
            raise ValueError("User block may only be specified "
                             "when creating a file")
        try:
            userblock_size = int(userblock_size)
        except (TypeError, ValueError):
            raise ValueError("User block size must be an integer")
        if fcpl is None:
            fcpl = h5p.create(h5p.FILE_CREATE)
        fcpl.set_userblock(userblock_size)

    if mode == 'r':
        fid = h5f.open(name, h5f.ACC_RDONLY, fapl=fapl)
    elif mode == 'r+':
        fid = h5f.open(name, h5f.ACC_RDWR, fapl=fapl)
    elif mode == 'w-':
        fid = h5f.create(name, h5f.ACC_EXCL, fapl=fapl, fcpl=fcpl)
    elif mode == 'w':
        fid = h5f.create(name, h5f.ACC_TRUNC, fapl=fapl, fcpl=fcpl)
    elif mode == 'a' or mode is None:
        try:
            fid = h5f.open(name, h5f.ACC_RDWR, fapl=fapl)
            try:
                existing_fcpl = fid.get_create_plist()
                if (userblock_size is not None and
                        existing_fcpl.get_userblock() != userblock_size):
                    raise ValueError("Requested userblock size (%d) does not match that of existing file (%d)" % (userblock_size, existing_fcpl.get_userblock()))
            except:
                fid.close()
                raise
        except IOError:
            fid = h5f.create(name, h5f.ACC_EXCL, fapl=fapl, fcpl=fcpl)
    else:
        raise ValueError("Invalid mode; must be one of r, r+, w, w-, a")

    return fid


class File(Group):

    """
        Represents an HDF5 file.
    """

    @property
    def attrs(self):
        """ Attributes attached to this object """
        # hdf5 complains that a file identifier is an invalid location for an
        # attribute. Instead of self, pass the root group to AttributeManager:
        import attrs
        return attrs.AttributeManager(self['/'])

    @property
    def filename(self):
        """File name on disk"""
        name = h5f.get_name(self.fid)
        try:
            return name.decode(sys.getfilesystemencoding())
        except (UnicodeError, LookupError):
            return name

    @property
    def driver(self):
        """Low-level HDF5 file driver used to open file"""
        drivers = {h5fd.SEC2: 'sec2', h5fd.STDIO: 'stdio',
                   h5fd.CORE: 'core', h5fd.FAMILY: 'family',
                   h5fd.WINDOWS: 'windows', h5fd.MPIO: 'mpio',
                   h5fd.MPIPOSIX: 'mpiposix'}
        return drivers.get(self.fid.get_access_plist().get_driver(), 'unknown')

    @property
    def mode(self):
        """ Python mode used to open file """
        return {h5f.ACC_RDONLY: 'r',
                h5f.ACC_RDWR: 'r+'}.get(self.fid.get_intent())

    @property
    def fid(self):
        """File ID (backwards compatibility) """
        return self.id

    @property
    def libver(self):
        """File format version bounds (2-tuple: low, high)"""
        bounds = self.id.get_access_plist().get_libver_bounds()
        return tuple(libver_dict_r[x] for x in bounds)

    @property
    def userblock_size(self):
        """ User block size (in bytes) """
        fcpl = self.fid.get_create_plist()
        return fcpl.get_userblock()


    if mpi and hdf5_version >= (1, 8, 9):

        @property
        def atomic(self):
            """ Set/get MPI-IO atomic mode 
            """
            return self.id.get_mpi_atomicity()

        @atomic.setter
        def atomic(self, value):
            self.id.set_mpi_atomicity(value)


    def __init__(self, name, mode=None, driver=None, 
                 libver=None, userblock_size=None, **kwds):
        """Create a new file object.

        See the h5py user guide for a detailed explanation of the options.

        name
            Name of the file on disk.  Note: for files created with the 'core'
            driver, HDF5 still requires this be non-empty.
        driver
            Name of the driver to use.  Legal values are None (default,
            recommended), 'core', 'sec2', 'stdio', 'mpio'.
        libver
            Library version bounds.  Currently only the strings 'earliest'
            and 'latest' are defined.
        userblock
            Desired size of user block.  Only allowed when creating a new
            file (mode w or w-).
        Additional keywords
            Passed on to the selected file driver.
        """
        if isinstance(name, _objects.ObjectID):
            fid = h5i.get_file_id(name)
        else:
            try:
                # If the byte string doesn't match the default
                # encoding, just pass it on as-is.  Note Unicode
                # objects can always be encoded.
                name = name.encode(sys.getfilesystemencoding())
            except (UnicodeError, LookupError):
                pass

            fapl = make_fapl(driver, libver, **kwds)
            fid = make_fid(name, mode, userblock_size, fapl)

        Group.__init__(self, fid)

    def close(self):
        """ Close the file.  All open objects become invalid """
        # TODO: find a way to square this with having issue 140
        # Not clearing shared state introduces a tiny memory leak, but
        # it goes like the number of files opened in a session.
        self.id.close()

    def flush(self):
        """ Tell the HDF5 library to flush its buffers.
        """
        h5f.flush(self.fid)

    def __enter__(self):
        return self

    def __exit__(self, *args):
        if self.id:
            self.close()

    def __repr__(self):
        if not self.id:
            r = u'<Closed HDF5 file>'
        else:
            # Filename has to be forced to Unicode if it comes back bytes
            # Mode is always a "native" string
            filename = self.filename
            if isinstance(filename, bytes):  # Can't decode fname
                filename = filename.decode('utf8', 'replace')
            r = u'<HDF5 file "%s" (mode %s)>' % (os.path.basename(filename),
                                                 self.mode)

        if py3:
            return r
        return r.encode('utf8')



#!/usr/bin/env python

from distutils.core import setup
from distutils.extension import Extension
from distutils.cmd import Command
from distutils.version import LooseVersion
import warnings
import sys, os
import os.path as op
from functools import reduce
import numpy

import configure   # Sticky-options configuration and version auto-detect

VERSION = '2.2.1'


# --- Autodetect Cython -------------------------------------------------------

HAVE_CYTHON = False
try:
    import Cython.Compiler.Version
    s = LooseVersion(Cython.Compiler.Version.version)
    if s.version[0:2] < [0, 13]:
        warnings.warn("Cython version %s too old; not used" % s.vstring)
        raise ImportError
    from Cython.Distutils import build_ext
    SUFFIX = '.pyx'
    HAVE_CYTHON = True
except ImportError:
    from distutils.command.build_ext import build_ext
    SUFFIX = '.c'

if sys.version_info[0] >= 3:
    # Shamelessly stolen from Cython 0.14
    import lib2to3.refactor
    from distutils.command.build_py \
         import build_py_2to3 as build_py
else:
    from distutils.command.build_py import build_py


# --- Support functions and "super-option" configuring ------------------------

def localpath(*args):
    return op.abspath(reduce(op.join, (op.dirname(__file__),)+args))

def configure_cython(settings, modules):
    """ Set up the Cython build environment.

    If configuration settings have changed since the last time Cython was
    run, re-write the file "config.pxi".  Also bump the utime of all the
    Cython modules to trigger a rebuild.
    """

    try:
        f = open(localpath('h5py/config.pxi'),'rb')
        oldcontents = f.read()
    except IOError:
        oldcontents = b""
    else:
        f.close()
    
    newcontents = """\
# This file is automatically generated by the h5py setup script.  Don't modify.

DEF MPI = %(mpi)s
DEF HDF5_VERSION = %(hdf5_version)s
"""
    newcontents %= settings
    newcontents = newcontents.encode('utf-8')

    # Only reconfigure and rebuild if settings have actually changed.
    if newcontents != oldcontents:
        with open(localpath('h5py/config.pxi'),'wb') as f:
            f.write(newcontents)
        for m in MODULES:
            os.utime(localpath('h5py',m+'.pyx'),None)


# --- Pre-compiling API generation --------------------------------------------

if not op.isfile(localpath('h5py','defs.pyx')):
    if not HAVE_CYTHON:
        raise ValueError("A modern version of Cython is required to build from source")
    import api_gen
    api_gen.run()


# --- Determine configuration settings ----------------------------------------

settings = configure.scrape_eargs()          # lowest priority
settings.update(configure.scrape_cargs())    # highest priority

HDF5 = settings.get('hdf5')
HDF5_VERSION = settings.get('hdf5_version')

MPI = settings.setdefault('mpi', False)
if MPI:
    if not HAVE_CYTHON:
        raise ValueError("Cython is required to compile h5py in MPI mode")
    try:
        import mpi4py
    except ImportError:
        raise ImportError("mpi4py is required to compile h5py in MPI mode")


# --- Configure Cython and create extensions ----------------------------------

if sys.platform.startswith('win'):
    COMPILER_SETTINGS = {
        'libraries'     : ['hdf5dll18','hdf5_hldll'],
        'include_dirs'  : [numpy.get_include(),  localpath('lzf'),
                           localpath('win_include')],
        'library_dirs'  : [],
        'define_macros' : [('H5_USE_16_API', None), ('_HDF5USEDLL_', None)]
    }
    if HDF5 is not None:
        COMPILER_SETTINGS['include_dirs'] += [op.join(HDF5, 'include')]
        COMPILER_SETTINGS['library_dirs'] += [op.join(HDF5, 'dll')]
else:
    COMPILER_SETTINGS = {
       'libraries'      : ['hdf5', 'hdf5_hl'],
       'include_dirs'   : [numpy.get_include(), localpath('lzf')],
       'library_dirs'   : [],
       'define_macros'  : [('H5_USE_16_API', None)]
    }
    if HDF5 is not None:
        COMPILER_SETTINGS['include_dirs'] += [op.join(HDF5, 'include')]
        COMPILER_SETTINGS['library_dirs'] += [op.join(HDF5, 'lib'), op.join(HDF5, 'lib64')]
    elif sys.platform == 'darwin':
        # putting here both macports and homebrew paths will generate
        # "ld: warning: dir not found" at the linking phase 
        COMPILER_SETTINGS['include_dirs'] += ['/opt/local/include'] # macports
        COMPILER_SETTINGS['library_dirs'] += ['/opt/local/lib']     # macports
        COMPILER_SETTINGS['include_dirs'] += ['/usr/local/include'] # homebrew
        COMPILER_SETTINGS['library_dirs'] += ['/usr/local/lib']     # homebrew
    elif sys.platform.startswith('freebsd'):
        COMPILER_SETTINGS['include_dirs'] += ['/usr/local/include'] # homebrew
        COMPILER_SETTINGS['library_dirs'] += ['/usr/local/lib']     # homebrew
    if MPI:
        COMPILER_SETTINGS['include_dirs'] += [mpi4py.get_include()]
    COMPILER_SETTINGS['runtime_library_dirs'] = [op.abspath(x) for x in COMPILER_SETTINGS['library_dirs']]

MODULES =  ['defs','_errors','_objects','_proxy', 'h5fd', 'h5z',
            'h5','h5i','h5r','utils',
            '_conv', 'h5t','h5s',
            'h5p',
            'h5d', 'h5a', 'h5f', 'h5g',
            'h5l', 'h5o',
            'h5ds', 'h5ac']

# No Cython, no point in configuring
if HAVE_CYTHON:     

    # Don't autodetect if version is manually given
    if HDF5_VERSION is None:    
        HDF5_VERSION = configure.autodetect(COMPILER_SETTINGS['library_dirs'])

    if HDF5_VERSION is None:
        HDF5_VERSION = (1, 8, 4)
        configure.printerr("HDF5 autodetection failed; building for 1.8.4+")
    
    settings['hdf5_version'] = HDF5_VERSION
    configure_cython(settings, MODULES)

else:
    configure.printerr("Cython not present; building for HDF5 1.8.4+")

EXTRA_SRC = {'h5z': [ localpath("lzf/lzf_filter.c"),
                      localpath("lzf/lzf/lzf_c.c"),
                      localpath("lzf/lzf/lzf_d.c")]}

def make_extension(module):
    sources = [op.join('h5py', module+SUFFIX)] + EXTRA_SRC.get(module, [])
    return Extension('h5py.'+module, sources, **COMPILER_SETTINGS)

EXTENSIONS = [make_extension(m) for m in MODULES]


# --- Custom distutils commands -----------------------------------------------

class test(Command):

    """Run the test suite."""

    description = "Run the test suite"

    user_options = [('verbosity=', 'V', 'set test report verbosity')]

    def initialize_options(self):
        self.verbosity = 0

    def finalize_options(self):
        try:
            self.verbosity = int(self.verbosity)
        except ValueError:
            raise ValueError('verbosity must be an integer.')

    def run(self):
        import sys
        py_version = sys.version_info[:2]
        if py_version == (2,7) or py_version >= (3,2):
            import unittest
        else:
            try:
                import unittest2 as unittest
            except ImportError:
                raise ImportError(
                    "unittest2 is required to run tests with python-%d.%d"
                    % py_version
                    )
        buildobj = self.distribution.get_command_obj('build')
        buildobj.run()
        oldpath = sys.path
        try:
            sys.path = [op.abspath(buildobj.build_lib)] + oldpath
            suite = unittest.TestLoader().discover(op.join(buildobj.build_lib,'h5py'))
            result = unittest.TextTestRunner(verbosity=self.verbosity+1).run(suite)
            if not result.wasSuccessful():
                sys.exit(1)
        finally:
            sys.path = oldpath



# --- Distutils setup and metadata --------------------------------------------

cls_txt = \
"""
Development Status :: 5 - Production/Stable
Intended Audience :: Developers
Intended Audience :: Information Technology
Intended Audience :: Science/Research
License :: OSI Approved :: BSD License
Programming Language :: Python
Topic :: Scientific/Engineering
Topic :: Database
Topic :: Software Development :: Libraries :: Python Modules
Operating System :: Unix
Operating System :: POSIX :: Linux
Operating System :: MacOS :: MacOS X
Operating System :: Microsoft :: Windows
"""

short_desc = "Read and write HDF5 files from Python"

long_desc = \
"""
The h5py package provides both a high- and low-level interface to the HDF5
library from Python. The low-level interface is intended to be a complete
wrapping of the HDF5 API, while the high-level component supports  access to
HDF5 files, datasets and groups using established Python and NumPy concepts.

A strong emphasis on automatic conversion between Python (Numpy) datatypes and
data structures and their HDF5 equivalents vastly simplifies the process of
reading and writing data from Python.

Supports HDF5 versions 1.8.3 and higher.  On Windows, HDF5 is included with
the installer.
"""

if os.name == 'nt':
    package_data = {'h5py': ['*.pyx', '*.dll']}
else:
    package_data = {'h5py': ['*.pyx']}

setup(
  name = 'h5py',
  version = VERSION,
  description = short_desc,
  long_description = long_desc,
  classifiers = [x for x in cls_txt.split("\n") if x],
  author = 'Andrew Collette',
  author_email = 'andrew dot collette at gmail dot com',
  maintainer = 'Andrew Collette',
  maintainer_email = 'andrew dot collette at gmail dot com',
  url = 'http://www.h5py.org',
  download_url = 'http://code.google.com/p/h5py/downloads/list',
  packages = ['h5py', 'h5py._hl', 'h5py._hl.tests', 'h5py.lowtest'],
  package_data = package_data,
  ext_modules = EXTENSIONS,
  requires = ['numpy (>=1.0.1)'],
  cmdclass = {'build_ext': build_ext, 'test': test, 'build_py':build_py}
)



# This file is part of h5py, a Python interface to the HDF5 library.
#
# http://www.h5py.org
#
# Copyright 2008-2013 Andrew Collette and contributors
#
# License:  Standard 3-clause BSD; see "license.txt" for full license terms
#           and contributor agreement.

"""
    Low-level HDF5 "H5G" group interface.
"""

# Compile-time imports
from h5d cimport DatasetID
from utils cimport emalloc, efree


def set_scale(DatasetID dset not None, char* dimname=''):
    """(DatasetID dset, STRING dimname)

    Convert dataset dset to a dimension scale, with optional name dimname.
    """
    H5DSset_scale(dset.id, dimname)

def is_scale(DatasetID dset not None):
    """(DatasetID dset)

    Determines whether dset is a dimension scale.
    """
    return <bint>(H5DSis_scale(dset.id))

def attach_scale(DatasetID dset not None, DatasetID dscale not None, unsigned
                 int idx):
    H5DSattach_scale(dset.id, dscale.id, idx)

def is_attached(DatasetID dset not None, DatasetID dscale not None,
                unsigned int idx):
    return <bint>(H5DSis_attached(dset.id, dscale.id, idx))

def detach_scale(DatasetID dset not None, DatasetID dscale not None,
                 unsigned int idx):
    H5DSdetach_scale(dset.id, dscale.id, idx)

def get_num_scales(DatasetID dset not None, unsigned int dim):
    return H5DSget_num_scales(dset.id, dim)

def set_label(DatasetID dset not None, unsigned int idx, char* label):
    H5DSset_label(dset.id, idx, label)

def get_label(DatasetID dset not None, unsigned int idx):
    cdef ssize_t size
    cdef char* label
    label = NULL

    size = H5DSget_label(dset.id, idx, NULL, 0)
    if size <= 0:
        return b''
    label = <char*>emalloc(sizeof(char)*(size+1))
    try:
        H5DSget_label(dset.id, idx, label, size+1)
        plabel = label
        return plabel
    finally:
        efree(label)

def get_scale_name(DatasetID dscale not None):
    cdef ssize_t namelen
    cdef char* name = NULL

    namelen = H5DSget_scale_name(dscale.id, NULL, 0)
    if namelen <= 0:
        return b''
    name = <char*>emalloc(sizeof(char)*(namelen+1))
    try:
        H5DSget_scale_name(dscale.id, name, namelen+1)
        pname = name
        return pname
    finally:
        efree(name)


cdef class _DimensionScaleVisitor:

    cdef object func
    cdef object retval

    def __init__(self, func):
        self.func = func
        self.retval = None


cdef herr_t cb_ds_iter(hid_t dset, unsigned int dim, hid_t scale, void* vis_in) except 2:

    cdef _DimensionScaleVisitor vis = <_DimensionScaleVisitor>vis_in

    # we did not retrieve the scale identifier using the normal machinery,
    # so we need to inc_ref it before using it to create a DatasetID.
    H5Iinc_ref(scale)
    vis.retval = vis.func(DatasetID.open(scale))

    if vis.retval is not None:
        return 1
    return 0


def iterate(DatasetID dset not None, unsigned int dim, object func,
            int startidx=0):
    """ (DatasetID loc, UINT dim, CALLABLE func, UINT startidx=0)
    => Return value from func

    Iterate a callable (function, method or callable object) over the
    members of a group.  Your callable shoutld have the signature::

        func(STRING name) => Result

    Returning None continues iteration; returning anything else aborts
    iteration and returns that value. Keywords:
    """
    if startidx < 0:
        raise ValueError("Starting index must be non-negative")

    cdef int i = startidx
    cdef _DimensionScaleVisitor vis = _DimensionScaleVisitor(func)

    H5DSiterate_scales(dset.id, dim, &i, <H5DS_iterate_t>cb_ds_iter, <void*>vis)

    return vis.retval



# This file is part of h5py, a Python interface to the HDF5 library.
#
# http://www.h5py.org
#
# Copyright 2008-2013 Andrew Collette and contributors
#
# License:  Standard 3-clause BSD; see "license.txt" for full license terms
#           and contributor agreement.

import numpy

from h5py import h5ds
from . import base
from .dataset import Dataset, readtime_dtype


class DimensionProxy(base.CommonStateObject):

    @property
    def label(self):
        #return h5ds.get_label(self._id, self._dimension)
        # Produces a segfault for a non-existent label (Fixed in hdf5-1.8.8).
        # Here is a workaround:
        try:
            dset = Dataset(self._id)
            return self._d(dset.attrs['DIMENSION_LABELS'][self._dimension])
        except (KeyError, IndexError):
            return ''
    @label.setter
    def label(self, val):
        h5ds.set_label(self._id, self._dimension, self._e(val))

    def __init__(self, id, dimension):
        self._id = id
        self._dimension = dimension

    def __hash__(self):
        return hash((type(self), self._id, self._dimension))

    def __eq__(self, other):
        return hash(self) == hash(other)

    def __iter__(self):
        for k in self.keys():
            yield k

    def __len__(self):
        return h5ds.get_num_scales(self._id, self._dimension)

    def __getitem__(self, item):
        if isinstance(item, int):
            scales = []
            def f(dsid):
                scales.append(Dataset(dsid))
            h5ds.iterate(self._id, self._dimension, f, 0)
            return scales[item]
        else:
            def f(dsid):
                if h5ds.get_scale_name(dsid) == self._e(item):
                    return Dataset(dsid)
            res = h5ds.iterate(self._id, self._dimension, f, 0)
            if res is None:
                raise KeyError('%s not found' % item)
            return res

    def attach_scale(self, dset):
        h5ds.attach_scale(self._id, dset.id, self._dimension)

    def detach_scale(self, dset):
        h5ds.detach_scale(self._id, dset.id, self._dimension)

    def items(self):
        scales = []
        def f(dsid):
            scales.append(dsid)
        h5ds.iterate(self._id, self._dimension, f, 0)
        return [
            (self._d(h5ds.get_scale_name(id)), Dataset(id))
            for id in scales
            ]

    def keys(self):
        return [key for (key, val) in self.items()]

    def values(self):
        return [val for (key, val) in self.items()]

    def __repr__(self):
        if not self._id:
            return "<Dimension of closed HDF5 dataset>"
        return ('<"%s" dimension %d of HDF5 dataset at %s>'
               % (self.label, self._dimension, id(self._id)))


class DimensionManager(base.DictCompat, base.CommonStateObject):

    """
    """

    def __init__(self, parent):
        """ Private constructor.
        """
        self._id = parent.id

    def __getitem__(self, index):
        """ Return a Dimension object
        """
        if index > len(self) - 1:
            raise IndexError('Index out of range')
        return DimensionProxy(self._id, index)

    def __len__(self):
        """ Number of dimensions associated with the dataset. """
        return len(Dataset(self._id).shape)

    def __iter__(self):
        """ Iterate over the dimensions. """
        for i in range(len(self)):
            yield self[i]

    def __repr__(self):
        if not self._id:
            return "<Dimensions of closed HDF5 dataset>"
        return "<Dimensions of HDF5 object at %s>" % id(self._id)

    def create_scale(self, dset, name=''):
        h5ds.set_scale(dset.id, self._e(name))



# This file is part of h5py, a Python interface to the HDF5 library.
#
# http://www.h5py.org
#
# Copyright 2008-2013 Andrew Collette and contributors
#
# License:  Standard 3-clause BSD; see "license.txt" for full license terms
#           and contributor agreement.

from h5py import _errors
_errors.silence_errors()

from h5py import _conv
_conv.register_converters()

from h5py import h5a, h5d, h5ds, h5f, h5fd, h5g, h5r, h5s, h5t, h5p, h5z

h5s.NULL = h5s._NULL  # NULL is a reserved name at the Cython layer
h5z._register_lzf()

from h5py.highlevel import *

from h5py.h5 import get_config
from h5py.h5r import Reference, RegionReference
from h5py.h5t import special_dtype, check_dtype

# Deprecated functions
from h5py.h5t import py_new_vlen as new_vlen
from h5py.h5t import py_get_vlen as get_vlen
from h5py.h5t import py_new_enum as new_enum
from h5py.h5t import py_get_enum as get_enum

from h5py import version

__version__ = version.version

__doc__ = \
"""
    This is the h5py package, a Python interface to the HDF5
    scientific data format.

    Version %s

    HDF5 %s
""" % (version.version, version.hdf5_version)


def enable_ipython_completer():
    import sys
    if 'IPython' in sys.modules:
        ip_running = False
        try:
            from IPython.core.interactiveshell import InteractiveShell
            ip_running = InteractiveShell.initialized()
        except ImportError:
            # support <ipython-0.11
            from IPython import ipapi as _ipapi
            ip_running = _ipapi.get() is not None
        except Exception:
            pass
        if ip_running:
            from . import ipy_completer
            return ipy_completer.load_ipython_extension()

    raise RuntimeError('completer must be enabled in active ipython session')

def run_tests(verbosity=1):
    import sys
    py_version = sys.version_info[:2]
    if py_version == (2,7) or py_version >= (3,2):
        import unittest
    else:
        try:
            import unittest2 as unittest
        except ImportError:
            raise ImportError(
                "unittest2 is required to run tests with python-%d.%d"
                % py_version
                )
    suite = unittest.TestLoader().discover('h5py')
    result = unittest.TextTestRunner(verbosity=verbosity).run(suite)



# This file is part of h5py, a Python interface to the HDF5 library.
#
# http://www.h5py.org
#
# Copyright 2008-2013 Andrew Collette and contributors
#
# License:  Standard 3-clause BSD; see "license.txt" for full license terms
#           and contributor agreement.

"""
    Filter API and constants.
"""

# === Public constants and data structures ====================================

FILTER_LZF = H5PY_FILTER_LZF

FILTER_ERROR    = H5Z_FILTER_ERROR
FILTER_NONE     = H5Z_FILTER_NONE
FILTER_ALL      = H5Z_FILTER_ALL
FILTER_DEFLATE  = H5Z_FILTER_DEFLATE
FILTER_SHUFFLE  = H5Z_FILTER_SHUFFLE
FILTER_FLETCHER32 = H5Z_FILTER_FLETCHER32
FILTER_SZIP     = H5Z_FILTER_SZIP
FILTER_NBIT     = H5Z_FILTER_NBIT
FILTER_SCALEOFFSET = H5Z_FILTER_SCALEOFFSET
FILTER_RESERVED = H5Z_FILTER_RESERVED
FILTER_MAX      = H5Z_FILTER_MAX

FLAG_DEFMASK    = H5Z_FLAG_DEFMASK
FLAG_MANDATORY  = H5Z_FLAG_MANDATORY
FLAG_OPTIONAL   = H5Z_FLAG_OPTIONAL
FLAG_INVMASK    = H5Z_FLAG_INVMASK
FLAG_REVERSE    = H5Z_FLAG_REVERSE
FLAG_SKIP_EDC   = H5Z_FLAG_SKIP_EDC

SZIP_ALLOW_K13_OPTION_MASK  = H5_SZIP_ALLOW_K13_OPTION_MASK   #1
SZIP_CHIP_OPTION_MASK       = H5_SZIP_CHIP_OPTION_MASK        #2
SZIP_EC_OPTION_MASK         = H5_SZIP_EC_OPTION_MASK          #4
SZIP_NN_OPTION_MASK         = H5_SZIP_NN_OPTION_MASK          #32
SZIP_MAX_PIXELS_PER_BLOCK   = H5_SZIP_MAX_PIXELS_PER_BLOCK    #32

SO_FLOAT_DSCALE = H5Z_SO_FLOAT_DSCALE
SO_FLOAT_ESCALE = H5Z_SO_FLOAT_ESCALE
SO_INT          = H5Z_SO_INT
SO_INT_MINBITS_DEFAULT = H5Z_SO_INT_MINBITS_DEFAULT

FILTER_CONFIG_ENCODE_ENABLED = H5Z_FILTER_CONFIG_ENCODE_ENABLED
FILTER_CONFIG_DECODE_ENABLED = H5Z_FILTER_CONFIG_DECODE_ENABLED

ERROR_EDC   = H5Z_ERROR_EDC
DISABLE_EDC = H5Z_DISABLE_EDC
ENABLE_EDC  = H5Z_ENABLE_EDC
NO_EDC      = H5Z_NO_EDC


# === Filter API  =============================================================


def filter_avail(int filter_code):
    """(INT filter_code) => BOOL

    Determine if the given filter is available to the library. The
    filter code should be one of:

    - FILTER_DEFLATE
    - FILTER_SHUFFLE
    - FILTER_FLETCHER32
    - FILTER_SZIP
    """
    return <bint>H5Zfilter_avail(<H5Z_filter_t>filter_code)


def get_filter_info(int filter_code):
    """(INT filter_code) => INT filter_flags

    Retrieve a bitfield with information about the given filter. The
    filter code should be one of:

    - FILTER_DEFLATE
    - FILTER_SHUFFLE
    - FILTER_FLETCHER32
    - FILTER_SZIP

    Valid bitmasks for use with the returned bitfield are:

    - FILTER_CONFIG_ENCODE_ENABLED
    - FILTER_CONFIG_DECODE_ENABLED
    """
    cdef unsigned int flags
    H5Zget_filter_info(<H5Z_filter_t>filter_code, &flags)
    return flags

def _register_lzf():
    register_lzf()









